@@ -12,9 +12,11 @@ from robocorp_ls_core.lsp import (
     TextDocumentTypedDict,
     CodeLensTypedDict,
     DocumentSymbolTypedDict,
+    PositionTypedDict,
 )
 from robotframework_ls.impl.protocols import IKeywordFound
 from robocorp_ls_core.watchdog_wrapper import IFSObserver
+import itertools
 
 
 log = get_logger(__name__)
@@ -46,6 +48,7 @@ class RobotFrameworkServerApi(PythonLanguageServer):
         self.libspec_manager = libspec_manager
         PythonLanguageServer.__init__(self, read_from, write_to)
         self._version = None
+        self._next_time = partial(next, itertools.count(0))
 
     @overrides(PythonLanguageServer._create_config)
     def _create_config(self) -> IConfig:
@@ -196,6 +199,15 @@ class RobotFrameworkServerApi(PythonLanguageServer):
         return func
 
     def _threaded_complete_all(self, doc_uri, line, col, monitor: IMonitor):
+        completion_context = self._create_completion_context(
+            doc_uri, line, col, monitor
+        )
+        if completion_context is None:
+            return []
+
+        return self._complete_from_completion_context(completion_context)
+
+    def _complete_from_completion_context(self, completion_context):
         from robotframework_ls.impl import section_name_completions
         from robotframework_ls.impl import keyword_completions
         from robotframework_ls.impl import variable_completions
@@ -207,12 +219,6 @@ class RobotFrameworkServerApi(PythonLanguageServer):
         )
         from robotframework_ls.impl import ast_utils
 
-        completion_context = self._create_completion_context(
-            doc_uri, line, col, monitor
-        )
-        if completion_context is None:
-            return []
-
         ret = section_name_completions.complete(completion_context)
         if not ret:
             ret.extend(filesystem_section_completions.complete(completion_context))
@@ -416,6 +422,27 @@ class RobotFrameworkServerApi(PythonLanguageServer):
 
         return code_lens(completion_context)
 
+    def m_resolve_code_lens(self, **code_lens: CodeLensTypedDict):
+        func = partial(self._threaded_resolve_code_lens, code_lens)
+        func = require_monitor(func)
+        return func
+
+    def _threaded_resolve_code_lens(
+        self, code_lens: CodeLensTypedDict, monitor: IMonitor
+    ) -> CodeLensTypedDict:
+        from robotframework_ls.impl.code_lens import code_lens_resolve
+
+        data = code_lens.get("data")
+        if not isinstance(data, dict):
+            return code_lens
+
+        doc_uri = data.get("uri")
+        completion_context = self._create_completion_context(doc_uri, 0, 0, monitor)
+        if completion_context is None:
+            return code_lens
+
+        return code_lens_resolve(completion_context, code_lens)
+
     def m_document_symbol(self, doc_uri: str):
         func = partial(self._threaded_document_symbol, doc_uri)
         func = require_monitor(func)
@@ -509,6 +536,196 @@ class RobotFrameworkServerApi(PythonLanguageServer):
             return {"resultId": None, "data": []}
         return {"resultId": None, "data": semantic_tokens_full(context)}
 
+    def m_monaco_completions_from_code_full(
+        self,
+        prefix: str = "",
+        full_code: str = "",
+        position=PositionTypedDict,
+        uri: str = "",
+        indent: str = "",
+    ):
+        func = partial(
+            self.threaded_monaco_completions_from_code_full,
+            prefix=prefix,
+            full_code=full_code,
+            position=position,
+            uri=uri,
+            indent=indent,
+        )
+        func = require_monitor(func)
+        return func
+
+    def threaded_monaco_completions_from_code_full(
+        self,
+        prefix: str,
+        full_code: str,
+        position: PositionTypedDict,
+        uri: str,
+        indent: str,
+        monitor: Optional[IMonitor] = None,
+    ):
+        from robotframework_ls.impl.robot_workspace import RobotDocument
+        from robotframework_ls.impl.completion_context import CompletionContext
+        from robocorp_ls_core.workspace import Document
+        from robotframework_ls.impl import section_completions
+        from robotframework_ls.impl import snippets_completions
+        from robotframework_ls.server_api.monaco_conversions import (
+            convert_to_monaco_completion,
+        )
+        from robotframework_ls.impl.completion_context import CompletionType
+
+        d = Document(uri, prefix)
+        last_line, _last_col = d.get_last_line_col()
+        line = last_line + position["line"]
+
+        col = position["character"]
+        col += len(indent)
+
+        document = RobotDocument(uri, full_code)
+        completion_context = CompletionContext(
+            document,
+            line,
+            col,
+            config=self.config,
+            monitor=monitor,
+            workspace=self.workspace,
+        )
+        completion_context.type = CompletionType.shell
+        completions = self._complete_from_completion_context(completion_context)
+        completions.extend(section_completions.complete(completion_context))
+        completions.extend(snippets_completions.complete(completion_context))
+
+        return {
+            "suggestions": [
+                convert_to_monaco_completion(
+                    c, line_delta=last_line, col_delta=len(indent), uri=uri
+                )
+                for c in completions
+            ]
+        }
+
+    def m_semantic_tokens_from_code_full(
+        self, prefix: str = "", full_code: str = "", indent: str = ""
+    ):
+        func = partial(
+            self.threaded_semantic_tokens_from_code_full,
+            prefix=prefix,
+            full_code=full_code,
+            indent=indent,
+        )
+        func = require_monitor(func)
+        return func
+
+    def threaded_semantic_tokens_from_code_full(
+        self,
+        prefix: str,
+        full_code: str,
+        indent: str,
+        monitor: Optional[IMonitor] = None,
+    ):
+        from robotframework_ls.impl.semantic_tokens import semantic_tokens_full_from_ast
+
+        try:
+            from robotframework_ls.impl.robot_workspace import RobotDocument
+
+            doc = RobotDocument("")
+            doc.source = full_code
+            ast = doc.get_ast()
+            data = semantic_tokens_full_from_ast(ast, monitor)
+            if not prefix:
+                return {"resultId": None, "data": data}
+
+            # We have to exclude the prefix from the coloring...
+
+            # debug info...
+            # import io
+            # from robotframework_ls.impl.semantic_tokens import decode_semantic_tokens
+            # stream = io.StringIO()
+            # decode_semantic_tokens(data, doc, stream)
+            # found = stream.getvalue()
+
+            prefix_doc = RobotDocument("")
+            prefix_doc.source = prefix
+            last_line, last_col = prefix_doc.get_last_line_col()
+
+            # Now we have the data from the full code, but we need to remove whatever
+            # we have in the prefix from the result...
+            ints_iter = iter(data)
+            line = 0
+            col = 0
+            new_data = []
+            indent_len = len(indent)
+            while True:
+                try:
+                    line_delta = next(ints_iter)
+                except StopIteration:
+                    break
+                col_delta = next(ints_iter)
+                token_len = next(ints_iter)
+                token_type = next(ints_iter)
+                token_modifier = next(ints_iter)
+                line += line_delta
+                if line_delta == 0:
+                    col += col_delta
+                else:
+                    col = col_delta
+
+                if line >= last_line:
+                    new_data.append(line - last_line)
+                    new_data.append(col_delta - indent_len)
+                    new_data.append(token_len)
+                    new_data.append(token_type)
+                    new_data.append(token_modifier)
+
+                    # Ok, now, we have to add the indent_len to all the
+                    # next lines
+                    while True:
+                        try:
+                            line_delta = next(ints_iter)
+                        except StopIteration:
+                            break
+                        col_delta = next(ints_iter)
+                        token_len = next(ints_iter)
+                        token_type = next(ints_iter)
+                        token_modifier = next(ints_iter)
+
+                        new_data.append(line_delta)
+                        if line_delta > 0:
+                            new_data.append(col_delta - indent_len)
+                        else:
+                            new_data.append(col_delta)
+                        new_data.append(token_len)
+                        new_data.append(token_type)
+                        new_data.append(token_modifier)
+
+                    break
+
+                # Approach changed so that we always have a new line
+                # i.e.:
+                # \n<indent><code>
+                #
+                # so, the condition below no longer applies.
+                # elif line == last_line and col >= last_col:
+                #     new_data.append(0)
+                #     new_data.append(col - last_col)
+                #     new_data.append(token_len)
+                #     new_data.append(token_type)
+                #     new_data.append(token_modifier)
+                #     new_data.extend(ints_iter)
+                #     break
+
+            # debug info...
+            # temp_stream = io.StringIO()
+            # temp_doc = RobotDocument("")
+            # temp_doc.source = full_code[len(prefix) :]
+            # decode_semantic_tokens(new_data, temp_doc, temp_stream)
+            # temp_found = temp_stream.getvalue()
+
+            return {"resultId": None, "data": new_data}
+        except:
+            log.exception("Error computing semantic tokens from code.")
+            return {"resultId": None, "data": []}
+
     def m_shutdown(self, **_kwargs):
         PythonLanguageServer.m_shutdown(self, **_kwargs)
         self.libspec_manager.dispose()
