@@ -1,13 +1,21 @@
 """
 Comments checkers
 """
+from codecs import (
+    BOM_UTF32_BE,
+    BOM_UTF32_LE,
+    BOM_UTF8,
+    BOM_UTF16_LE,
+    BOM_UTF16_BE
+)
+
 from robocop.checkers import RawFileChecker, VisitorChecker
 from robocop.rules import RuleSeverity
 from robocop.utils import IS_RF4
 
 
 class CommentChecker(VisitorChecker):
-    """ Checker for content of comments. It detects invalid comments or leftover `todo` or `fixme` in code. """
+    """ Checker for comments content. It detects invalid comments or leftovers like `todo` or `fixme` in the code. """
     rules = {
         "0701": (
             "todo-in-comment",
@@ -70,13 +78,31 @@ class IgnoredDataChecker(RawFileChecker):
             "ignored-data",
             "Ignored data found in file",
             RuleSeverity.WARNING
+        ),
+        "0705": (
+            "bom-encoding-in-file",
+            "This file contains BOM (Byte Order Mark) encoding not supported by Robot Framework",
+            RuleSeverity.WARNING
         )
     }
+    BOM = [
+        BOM_UTF32_BE,
+        BOM_UTF32_LE,
+        BOM_UTF8,
+        BOM_UTF16_LE,
+        BOM_UTF16_BE
+    ]
+
+    def __init__(self):
+        self.is_bom = False
+        super().__init__()
 
     def parse_file(self):
+        self.is_bom = False
         if self.lines is not None:
             self._parse_lines(self.lines)
         else:
+            self.detect_bom(self.source)
             with open(self.source) as file:
                 self._parse_lines(file)
 
@@ -88,6 +114,16 @@ class IgnoredDataChecker(RawFileChecker):
     def check_line(self, line, lineno):
         if line.startswith('***'):
             return True
-        if not line.startswith('***') and not line.startswith('# robocop:'):
+        elif not line.startswith('# robocop:'):
+            if lineno == 1 and self.is_bom:
+                # if it's BOM encoded file, first line can be ignored
+                return '***' in line
             self.report("ignored-data", lineno=lineno, col=0)
             return True
+
+    def detect_bom(self, source):
+        with open(source, 'rb') as raw_file:
+            first_four = raw_file.read(4)
+            self.is_bom = any(first_four.startswith(bom_marker) for bom_marker in IgnoredDataChecker.BOM)
+            if self.is_bom:
+                self.report("bom-encoding-in-file", lineno=1, col=0)
